import axios, { AxiosInstance, AxiosError } from 'axios';
import { boot } from 'quasar/wrappers';
import config from '../config';
import { Store } from 'vuex';
import { StateInterface } from 'src/store';
import { Notify } from 'quasar';

declare module 'vue/types/vue' {
  interface Vue {
    $axios: AxiosInstance;
  }
}

export default boot<Store<StateInterface>>(({ Vue, store }) => {
  // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
  Vue.prototype.$axios = axios;
  axios.defaults.baseURL = config.baseURL;

  /***
   * Intercept requests and insert Token if available
   */
  axios.interceptors.request.use(config => {
    const session = store.state.session.currentSession;
    if (session?.token) {
      config.headers = { Authorization: 'Bearer ' + session.token };
    }
    return config;
  });

  /***
   * Intercept responses
   *   - filter 401 --> logout
   *   - filter timeout or 502 --> backendOffline
   */
  axios.interceptors.response.use(
    response => response,
    error => {
      if (error) {
        const e = <AxiosError>error;
        if (
          e.code === 'ECONNABORTED' ||
          (e.response && e.response.status === 502)
        ) {
          store.commit('session/setOffline', true);
        } else if (e.response && e.response.status == 401) {
          return store.dispatch('session/clearCurrent');
        } else {
          Notify.create({
            type: 'error',
            message: 'Eine Aktion konnte nicht durchgeführt werden.',
            group: false
          });
        }
      }
      return Promise.reject(error);
    }
  );
});

export { axios };
